@@ -1,11 +1,12 @@
+{-# OPTIONS_GHC -cpp #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.PackageDescription
--- Copyright   :  Isaac Jones 2003-2004
+-- Copyright   :  Isaac Jones 2003-2005
 -- 
 -- Maintainer  :  Isaac Jones <ijones@syntaxpolice.org>
 -- Stability   :  alpha
--- Portability :  
+-- Portability :  portable
 --
 -- Package description and parsing.
 
@@ -41,15 +42,15 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. -}
 
 module Distribution.PackageDescription (
         -- * Package descriptions
-	PackageDescription(..),
-	emptyPackageDescription,
+        PackageDescription(..),
+        emptyPackageDescription,
         readPackageDescription,
-	parseDescription,
+        parseDescription,
         StanzaField(..),
         LineNo,
         basicStanzaFields,
         writePackageDescription,
-	showPackageDescription,
+        showPackageDescription,
         sanityCheckPackage, errorOut,
         setupMessage,
         Library(..),
@@ -63,40 +64,59 @@ module Distribution.PackageDescription (
         BuildInfo(..),
         emptyBuildInfo,
         -- ** Supplementary build information
-	HookedBuildInfo,
-	emptyHookedBuildInfo,
-	readHookedBuildInfo,
-	parseHookedBuildInfo,
-	writeHookedBuildInfo,
-	showHookedBuildInfo,        
-	updatePackageDescription,
+        HookedBuildInfo,
+        emptyHookedBuildInfo,
+        readHookedBuildInfo,
+        parseHookedBuildInfo,
+        writeHookedBuildInfo,
+        showHookedBuildInfo,        
+        updatePackageDescription,
         -- * Utilities
         ParseResult(..),
-        PError, showError,
+        PError, PWarning, showError,
         hcOptions,
+        autogenModuleName,
+        haddockName,
+
+
+
 
   ) where
 
-import Control.Monad(liftM, foldM, when, unless)
+import Control.Monad(liftM, foldM, when)
 import Data.Char
-import Data.Maybe(fromMaybe, fromJust, isNothing, catMaybes)
+import Data.Maybe(fromMaybe, isNothing, catMaybes)
+import Data.List (nub,lookup)
 import Text.PrettyPrint.HughesPJ
 import System.Directory(doesFileExist)
+import System.Environment(getProgName)
+import System.IO(hPutStrLn, stderr)
+import System.Exit
 
 import Distribution.ParseUtils
 import Distribution.Package(PackageIdentifier(..),showPackageId,
-			    parsePackageName)
-import Distribution.Version(Version(..), VersionRange(..),
-                            showVersion, parseVersion)
+                            parsePackageName)
+import Distribution.Version(Version(..), VersionRange(..), withinRange,
+                            showVersion, parseVersion, showVersionRange, parseVersionRange)
 import Distribution.License(License(..))
 import Distribution.Version(Dependency(..))
-import Distribution.Extension(Extension(..))
-import Distribution.Setup(CompilerFlavor(..))
-import Distribution.Simple.Utils(currentDir, die)
+import Distribution.Compiler(CompilerFlavor(..))
+import Distribution.Simple.Utils(currentDir, die, dieWithLocation, warn)
+import Language.Haskell.Extension(Extension(..))
 
 import Distribution.Compat.ReadP as ReadP hiding (get)
+import Distribution.Compat.FilePath(joinFileExt)
 
--- | This data type is the internal representation of the file @pkg.descr@.
+
+
+
+
+
+-- |Fix. Figure out a way to get this from .cabal file
+cabalVersion :: Version
+cabalVersion = Version [1,1,4] []
+
+-- | This data type is the internal representation of the file @pkg.cabal@.
 -- It contains two kinds of information about the package: information
 -- which is needed for all packages, such as the package name and version, and 
 -- information which is needed for the simple build system only, such as 
@@ -104,30 +124,36 @@ import Distribution.Compat.ReadP as ReadP hiding (get)
 -- 
 data PackageDescription
     =  PackageDescription {
-	-- the following are required by all packages:
-	package        :: PackageIdentifier,
+        -- the following are required by all packages:
+        package        :: PackageIdentifier,
         license        :: License,
         licenseFile    :: FilePath,
         copyright      :: String,
         maintainer     :: String,
-	author         :: String,
+        author         :: String,
         stability      :: String,
-	testedWith     :: [(CompilerFlavor,VersionRange)],
-	homepage       :: String,
-	pkgUrl         :: String,
-	synopsis       :: String,
-	description    :: String,
-	category       :: String,
+        testedWith     :: [(CompilerFlavor,VersionRange)],
+        homepage       :: String,
+        pkgUrl         :: String,
+        synopsis       :: String, -- ^A one-line summary of this package
+        description    :: String, -- ^A more verbose description of this package
+        category       :: String,
         buildDepends   :: [Dependency],
-	-- components
+        descCabalVersion :: VersionRange, -- ^If this package depends on a specific version of Cabal, give that here.
+        -- components
         library        :: Maybe Library,
-        executables    :: [Executable]
+        executables    :: [Executable],
+        dataFiles      :: [FilePath],
+        extraSrcFiles  :: [FilePath],
+        extraTmpFiles  :: [FilePath]
     }
     deriving (Show, Read, Eq)
 
-data Library = Library { exposedModules    :: [String],
-                         libBuildInfo      :: BuildInfo }
-             deriving (Show, Eq, Read)
+data Library = Library {
+        exposedModules    :: [String],
+        libBuildInfo      :: BuildInfo
+    }
+    deriving (Show, Eq, Read)
 
 emptyLibrary :: Library
 emptyLibrary = Library [] emptyBuildInfo
@@ -137,26 +163,30 @@ emptyPackageDescription
     =  PackageDescription {package      = PackageIdentifier "" (Version [] []),
                       license      = AllRightsReserved,
                       licenseFile  = "",
+                      descCabalVersion = AnyVersion,
                       copyright    = "",
                       maintainer   = "",
-		      author       = "",
+                      author       = "",
                       stability    = "",
-		      testedWith   = [],
+                      testedWith   = [],
                       buildDepends = [],
-		      homepage     = "",
-		      pkgUrl       = "",
-		      synopsis     = "",
-		      description  = "",
-		      category     = "",
+                      homepage     = "",
+                      pkgUrl       = "",
+                      synopsis     = "",
+                      description  = "",
+                      category     = "",
                       library      = Nothing,
-                      executables  = []
+                      executables  = [],
+                      dataFiles    = [],
+                      extraSrcFiles = [],
+                      extraTmpFiles = []
                      }
 
 -- |Get all the module names from the libraries in this package
 libModules :: PackageDescription -> [String]
 libModules PackageDescription{library=lib}
-    = (maybe [] exposedModules lib)
-      ++ (maybe [] (otherModules . libBuildInfo) lib)
+    = maybe [] exposedModules lib
+       ++ maybe [] (otherModules . libBuildInfo) lib
 
 -- |Get all the module names from the exes in this package
 exeModules :: PackageDescription -> [String]
@@ -167,39 +197,48 @@ exeModules PackageDescription{executables=execs}
 hasLibs :: PackageDescription -> Bool
 hasLibs p = maybe False (buildable . libBuildInfo) (library p)
 
+-- |'Maybe' version of 'hasLibs'
+maybeHasLibs :: PackageDescription -> Maybe Library
+maybeHasLibs p =
+   library p >>= (\lib -> toMaybe (buildable (libBuildInfo lib)) lib)
+
 -- Consider refactoring into executable and library versions.
 data BuildInfo = BuildInfo {
         buildable         :: Bool,      -- ^ component is buildable here
         ccOptions         :: [String],  -- ^ options for C compiler
         ldOptions         :: [String],  -- ^ options for linker
-        frameworks        :: [String],
+        frameworks        :: [String], -- ^support frameworks for Mac OS X
         cSources          :: [FilePath],
-        hsSourceDir       :: FilePath,
-	otherModules      :: [String],
+        hsSourceDirs      :: [FilePath], -- ^ where to look for the haskell module hierarchy
+        otherModules      :: [String], -- ^ non-exposed or non-main modules
         extensions        :: [Extension],
-        extraLibs         :: [String],
+        extraLibs         :: [String], -- ^ what libraries to link with when compiling a program that uses your package
         extraLibDirs      :: [String],
-        includeDirs       :: [FilePath],
-        includes          :: [FilePath],
-        options           :: [(CompilerFlavor,[String])]
+        includeDirs       :: [FilePath], -- ^directories to find .h files
+        includes          :: [FilePath], -- ^ The .h files to be found in includeDirs
+	installIncludes   :: [FilePath], -- ^ .h files to install with the package
+        options           :: [(CompilerFlavor,[String])],
+        ghcProfOptions       :: [String]
     }
     deriving (Show,Read,Eq)
 
 emptyBuildInfo :: BuildInfo
 emptyBuildInfo = BuildInfo {
-		      buildable         = True,
-		      ccOptions         = [],
-		      ldOptions         = [],
-		      frameworks        = [],
-		      cSources          = [],
-		      hsSourceDir       = currentDir,
-		      otherModules      = [],
+                      buildable         = True,
+                      ccOptions         = [],
+                      ldOptions         = [],
+                      frameworks        = [],
+                      cSources          = [],
+                      hsSourceDirs      = [currentDir],
+                      otherModules      = [],
                       extensions        = [],
                       extraLibs         = [],
                       extraLibDirs      = [],
                       includeDirs       = [],
                       includes          = [],
-                      options           = []
+                      installIncludes   = [],
+                      options           = [],
+                      ghcProfOptions       = []
                      }
 
 data Executable = Executable {
@@ -234,9 +273,8 @@ emptyHookedBuildInfo = (Nothing, [])
 -- |If the package description has a library section, call the given
 --  function with the library build info as argument.
 withLib :: PackageDescription -> a -> (Library -> IO a) -> IO a
-withLib pkg_descr a f = if hasLibs pkg_descr
-                        then f (fromJust (library pkg_descr))
-                        else return a
+withLib pkg_descr a f =
+   maybe (return a) f (maybeHasLibs pkg_descr)
 
 setupMessage :: String -> PackageDescription -> IO ()
 setupMessage msg pkg_descr = 
@@ -260,14 +298,18 @@ updatePackageDescription (mb_lib_bi, exe_bi) p
        --won't build anyway.  add to sanity checker?
       updateLibrary (Just bi) Nothing     = Just emptyLibrary{libBuildInfo=bi}
 
-      updateExecutables :: [(String, BuildInfo)] -> [Executable] -> [Executable]
+      updateExecutables :: [(String, BuildInfo)] -- ^[(exeName, new buildinfo)]
+                        -> [Executable]          -- ^list of executables to update
+                        -> [Executable]          -- ^list with exeNames updated
       updateExecutables exe_bi' executables' = foldr updateExecutable executables' exe_bi'
       
-      updateExecutable :: (String, BuildInfo) -> [Executable] -> [Executable]
+      updateExecutable :: (String, BuildInfo) -- ^(exeName, new buildinfo)
+                       -> [Executable]        -- ^list of executables to update
+                       -> [Executable]        -- ^libst with exeName updated
       updateExecutable _                 []         = []
       updateExecutable exe_bi'@(name,bi) (exe:exes)
         | exeName exe == name = exe{buildInfo = unionBuildInfo bi (buildInfo exe)} : exes
-        | otherwise           = updateExecutable exe_bi' exes
+        | otherwise           = exe : updateExecutable exe_bi' exes
 
 unionBuildInfo :: BuildInfo -> BuildInfo -> BuildInfo
 unionBuildInfo b1 b2
@@ -276,31 +318,35 @@ unionBuildInfo b1 b2
          ldOptions         = combine ldOptions,
          frameworks        = combine frameworks,
          cSources          = combine cSources,
-         hsSourceDir       = override hsSourceDir "hs-source-dir",
+         hsSourceDirs      = combine hsSourceDirs,
          otherModules      = combine otherModules,
          extensions        = combine extensions,
          extraLibs         = combine extraLibs,
          extraLibDirs      = combine extraLibDirs,
          includeDirs       = combine includeDirs,
          includes          = combine includes,
+         installIncludes   = combine installIncludes,
          options           = combine options
         }
       where 
       combine :: (Eq a) => (BuildInfo -> [a]) -> [a]
-      combine f = f b1 ++ f b2
-      override :: (Eq a) => (BuildInfo -> a) -> String -> a
-      override f s
-	| v1 == def = v2
-	| v2 == def = v1
-	| otherwise = error $ "union: Two non-empty fields found in union attempt: " ++ s
-        where v1 = f b1
-	      v2 = f b2
-	      def = f emptyBuildInfo
+      combine f = nub $ f b1 ++ f b2
 
 -- |Select options for a particular Haskell compiler.
 hcOptions :: CompilerFlavor -> [(CompilerFlavor, [String])] -> [String]
 hcOptions hc hc_opts = [opt | (hc',opts) <- hc_opts, hc' == hc, opt <- opts]
 
+-- |The name of the auto-generated module associated with a package
+autogenModuleName :: PackageDescription -> String
+autogenModuleName pkg_descr =
+    "Paths_" ++ map fixchar (pkgName (package pkg_descr))
+  where fixchar '-' = '_'
+        fixchar c   = c
+
+haddockName :: PackageDescription -> FilePath
+haddockName pkg_descr =
+   joinFileExt (pkgName (package pkg_descr)) "haddock"
+
 -- ------------------------------------------------------------
 -- * Parsing & Pretty printing
 -- ------------------------------------------------------------
@@ -320,8 +366,11 @@ basicStanzaFields =
                            text                   parsePackageName
                            (pkgName . package)    (\name pkg -> pkg{package=(package pkg){pkgName=name}})
  , simpleField reqNameVersion
-                           (text . showVersion)   parseVersion 
+                           (text . showVersion)   parseVersion
                            (pkgVersion . package) (\ver pkg -> pkg{package=(package pkg){pkgVersion=ver}})
+ , simpleField "cabal-version"
+                           (text . showVersionRange) parseVersionRange
+                           descCabalVersion       (\v pkg -> pkg{descCabalVersion=v})
  , simpleField "license"
                            (text . show)          parseLicenseQ
                            license                (\l pkg -> pkg{license=l})
@@ -361,12 +410,12 @@ basicStanzaFields =
  , listField "tested-with"
                            showTestedWith         parseTestedWithQ
                            testedWith             (\val pkg -> pkg{testedWith=val})
-
- , listField   "exposed-modules"
-                           text               parseModuleNameQ
-                           (\p -> maybe [] exposedModules (library p))
-                           (\xs    pkg -> let lib = fromMaybe emptyLibrary (library pkg) in
-                                              pkg{library = Just lib{exposedModules=xs}})
+ , listField "data-files"  showFilePath           parseFilePathQ
+                           dataFiles              (\val pkg -> pkg{dataFiles=val})
+ , listField "extra-source-files" showFilePath    parseFilePathQ
+                           extraSrcFiles          (\val pkg -> pkg{extraSrcFiles=val})
+ , listField "extra-tmp-files" showFilePath       parseFilePathQ
+                           extraTmpFiles          (\val pkg -> pkg{extraTmpFiles=val})
  ]
 
 executableStanzaFields :: [StanzaField Executable]
@@ -385,13 +434,13 @@ binfoFields =
                            (text . show)      parseReadS
                            buildable          (\val binfo -> binfo{buildable=val})
  , listField "cc-options"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            ccOptions          (\val binfo -> binfo{ccOptions=val})
  , listField "ld-options"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            ldOptions          (\val binfo -> binfo{ldOptions=val})
  , listField "frameworks"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            frameworks         (\val binfo -> binfo{frameworks=val})
  , listField   "c-sources"
                            showFilePath       parseFilePathQ
@@ -400,7 +449,7 @@ binfoFields =
                            (text . show)      parseExtensionQ
                            extensions         (\exts  binfo -> binfo{extensions=exts})
  , listField   "extra-libraries"
-                           showToken	      parseTokenQ
+                           showToken          parseTokenQ
                            extraLibs          (\xs    binfo -> binfo{extraLibs=xs})
  , listField   "extra-lib-dirs"
                            showFilePath       parseFilePathQ
@@ -408,23 +457,32 @@ binfoFields =
  , listField   "includes"
                            showFilePath       parseFilePathQ
                            includes           (\paths binfo -> binfo{includes=paths})
+ , listField   "install-includes"
+                           showFilePath       parseFilePathQ
+                           includes           (\paths binfo -> binfo{installIncludes=paths})
  , listField   "include-dirs"
                            showFilePath       parseFilePathQ
                            includeDirs        (\paths binfo -> binfo{includeDirs=paths})
- , simpleField "hs-source-dir"
+ , listField   "hs-source-dirs"
                            showFilePath       parseFilePathQ
-                           hsSourceDir        (\path  binfo -> binfo{hsSourceDir=path})
+                           hsSourceDirs       (\paths binfo -> binfo{hsSourceDirs=paths})
  , listField   "other-modules"         
                            text               parseModuleNameQ
                            otherModules       (\val binfo -> binfo{otherModules=val})
+ , listField   "ghc-prof-options"         
+                           text               parseTokenQ
+                           ghcProfOptions        (\val binfo -> binfo{ghcProfOptions=val})
  , optsField   "ghc-options"  GHC
                            options            (\path  binfo -> binfo{options=path})
  , optsField   "hugs-options" Hugs
                            options            (\path  binfo -> binfo{options=path})
  , optsField   "nhc-options"  NHC
                            options            (\path  binfo -> binfo{options=path})
+ , optsField   "jhc-options"  JHC
+                           options            (\path  binfo -> binfo{options=path})
  ]
 
+
 -- --------------------------------------------
 -- ** Parsing
 
@@ -436,8 +494,12 @@ readAndParseFile parser fpath = do
   when (not exists) (die $ "Error Parsing: file \"" ++ fpath ++ "\" doesn't exist. Cannot continue.")
   str <- readFile fpath
   case parser str of
-    ParseFailed e -> error (showError e) -- FIXME
-    ParseOk x -> return x
+    ParseFailed e -> do
+        let (lineNo, message) = locatedErrorMsg e
+        dieWithLocation fpath lineNo message
+    ParseOk ws x -> do
+        mapM_ warn ws
+        return x
 
 -- |Parse the given package file.
 readPackageDescription :: FilePath -> IO PackageDescription
@@ -455,43 +517,49 @@ parseDescription inp = do (st:sts) <- splitStanzas inp
         parseBasicStanza ((StanzaField name _ set):fields) pkg (lineNo, f, val)
           | name == f = set lineNo val pkg
           | otherwise = parseBasicStanza fields pkg (lineNo, f, val)
-        parseBasicStanza [] pkg (lineNo, f, val) = do
-          let lib = fromMaybe emptyLibrary (library pkg)
-	  bi <- parseBInfoField binfoFields (libBuildInfo lib) (lineNo, f, val)
-          return pkg{library=Just lib{libBuildInfo=bi}}
+          {-     
+     , listField   "exposed-modules"
+                           text               parseModuleNameQ
+                           (\p -> maybe [] exposedModules (library p))
+                           (\xs    pkg -> let lib = fromMaybe emptyLibrary (library pkg) in
+                                              pkg{library = Just lib{exposedModules=xs}})
+-}
+        parseBasicStanza [] pkg (lineNo, f, val)
+          | "exposed-modules" == f = do
+               mods <- runP lineNo f (parseOptCommaList parseModuleNameQ) val
+               return pkg{library=Just lib{exposedModules=mods}}
+          | otherwise = do
+               bi <- parseBInfoField binfoFields (libBuildInfo lib) (lineNo, f, val)
+               return pkg{library=Just lib{libBuildInfo=bi}}
+          where
+            lib = fromMaybe emptyLibrary (library pkg)
 
-        parseExecutableStanza st@((_, "executable",eName):_) =
+        parseExecutableStanza st@((lineNo, "executable",eName):_) =
           case lookupField "main-is" st of
-	    Just (_,_) -> foldM (parseExecutableField executableStanzaFields) emptyExecutable st
-	    Nothing           -> fail $ "No 'Main-Is' field found for " ++ eName ++ " stanza"
+            Just (_,_) -> foldM (parseExecutableField executableStanzaFields) emptyExecutable st
+            Nothing    -> syntaxError lineNo $ "No 'Main-Is' field found for " ++ eName ++ " stanza"
         parseExecutableStanza ((lineNo, f,_):_) = 
-          myError lineNo $ "'Executable' stanza starting with field '" ++ f ++ "'"
+          syntaxError lineNo $ "'Executable' stanza starting with field '" ++ f ++ "'"
         parseExecutableStanza _ = error "This shouldn't happen!"
 
         parseExecutableField ((StanzaField name _ set):fields) exe (lineNo, f, val)
-	  | name == f = set lineNo val exe
-	  | otherwise = parseExecutableField fields exe (lineNo, f, val)
-	parseExecutableField [] exe (lineNo, f, val) = do
-	  binfo <- parseBInfoField binfoFields (buildInfo exe) (lineNo, f, val)
+          | name == f = set lineNo val exe
+          | otherwise = parseExecutableField fields exe (lineNo, f, val)
+        parseExecutableField [] exe (lineNo, f, val) = do
+          binfo <- parseBInfoField binfoFields (buildInfo exe) (lineNo, f, val)
           return exe{buildInfo=binfo}
 
         -- ...
         lookupField :: String -> Stanza -> Maybe (LineNo,String)
+        lookupField x sts = lookup x (map (\(n,f,v) -> (f,(n,v))) sts)
 
-        lookupField x ((n,f,v):st)
-          | x == f      = Just (n,v)
-          | otherwise   = lookupField x st
-        lookupField _ [] = Nothing
 
 parseHookedBuildInfo :: String -> ParseResult HookedBuildInfo
 parseHookedBuildInfo inp = do
   stanzas@(mLibStr:exes) <- splitStanzas inp
   mLib <- parseLib mLibStr
-  case mLib of
-    Nothing      -> do biExes <- mapM parseExe stanzas
-                       return (Nothing, biExes)
-    lib@(Just _) -> do biExes <- mapM parseExe exes
-                       return (lib, biExes)
+  biExes <- mapM parseExe (maybe stanzas (const exes) mLib)
+  return (mLib, biExes)
   where
     parseLib :: Stanza -> ParseResult (Maybe BuildInfo)
     parseLib (bi@((_, inFieldName, _):_))
@@ -502,17 +570,20 @@ parseHookedBuildInfo inp = do
         | map toLower inFieldName == "executable"
             = do bis <- parseBI bi
                  return (mName, bis)
-        | otherwise = myError lineNo "expecting 'executable' at top of stanza"
-    parseExe [] = myError 0 "error in parsing buildinfo file. Expected executable stanza"
+        | otherwise = syntaxError lineNo "expecting 'executable' at top of stanza"
+    parseExe [] = syntaxError 0 "error in parsing buildinfo file. Expected executable stanza"
     parseBI :: Stanza -> ParseResult BuildInfo
     parseBI st = foldM (parseBInfoField binfoFields) emptyBuildInfo st
 
 parseBInfoField :: [StanzaField a] -> a -> (LineNo, String, String) -> ParseResult a
 parseBInfoField ((StanzaField name _ set):fields) binfo (lineNo, f, val)
-	  | name == f = set lineNo val binfo
-	  | otherwise = parseBInfoField fields binfo (lineNo, f, val)
-parseBInfoField [] _ (lineNo, f, _) =
-	  myError lineNo $ "Unknown field '" ++ f ++ "'"
+          | name == f = set lineNo val binfo
+          | otherwise = parseBInfoField fields binfo (lineNo, f, val)
+-- ignore "x-" extension fields without a warning
+parseBInfoField [] binfo (lineNo, 'x':'-':f, _) = return binfo
+parseBInfoField [] binfo (lineNo, f, _) = do
+          warning $ "Unknown field '" ++ f ++ "'"
+          return binfo
 
 -- --------------------------------------------
 -- ** Pretty printing
@@ -525,7 +596,9 @@ showPackageDescription pkg = render $
   ppFields pkg basicStanzaFields $$
   (case library pkg of
      Nothing  -> empty
-     Just lib -> ppFields (libBuildInfo lib) binfoFields) $$
+     Just lib -> 
+        text "exposed-modules" <> colon <+> fsep (punctuate comma (map text (exposedModules lib))) $$
+        ppFields (libBuildInfo lib) binfoFields) $$
   vcat (map ppExecutable (executables pkg))
   where
     ppExecutable exe =
@@ -558,6 +631,7 @@ showHookedBuildInfo (mb_lib_bi, ex_bi) = render $
     ppFields bi ((StanzaField name get _):flds) =
            ppField name (get bi) $$ ppFields bi flds
 
+
 -- ------------------------------------------------------------
 -- * Sanity Checking
 -- ------------------------------------------------------------
@@ -580,19 +654,27 @@ sanityCheckPackage pkg_descr
                       "Package is copyright All Rights Reserved"
           noLicenseFile = checkSanity (null $ licenseFile pkg_descr)
                           "No license-file field."
+          goodCabal = let v = (descCabalVersion pkg_descr)
+                          in checkSanity (not $ cabalVersion  `withinRange` v)
+                                 ("This package requires Cabal verion: " ++ (showVersionRange v) ++ ".")
 
          in return $ (catMaybes [nothingToDo, noModules,
                                  allRights, noLicenseFile]
-                     ,catMaybes $ libSane:(checkMissingFields pkg_descr))
+                     ,catMaybes $ libSane:goodCabal:(checkMissingFields pkg_descr))
 
 -- |Output warnings and errors. Exit if any errors.
 errorOut :: [String]  -- ^Warnings
          -> [String]  -- ^errors
          -> IO ()
 errorOut warnings errors = do
-  mapM (putStrLn . ("Warning: " ++)) warnings
-  mapM (putStrLn . ("Error: " ++))   errors
-  unless (null errors) (error "Errors detected. See above.")
+  mapM warn warnings
+  when (not (null errors)) $ do
+    pname <- getProgName
+    mapM (hPutStrLn stderr . ((pname ++ ": Error: ") ++)) errors
+    exitWith (ExitFailure 1)
+
+toMaybe :: Bool -> a -> Maybe a
+toMaybe b x = if b then Just x else Nothing
 
 checkMissingFields :: PackageDescription -> [Maybe String]
 checkMissingFields pkg_descr = 
@@ -603,28 +685,212 @@ checkMissingFields pkg_descr =
                        -> String -- Name of field
                        -> Maybe String -- error message
           missingField f n
-              = if null (f pkg_descr)
-                 then Just $ "Missing field: " ++ n
-                 else Nothing
+              = toMaybe (null (f pkg_descr)) ("Missing field: " ++ n)
 
 sanityCheckLib :: Maybe Library -> Maybe String
-sanityCheckLib Nothing  = Nothing
-sanityCheckLib (Just l)
-    = if null $ exposedModules l
-      then Just "Non-empty library, but empty exposed modules list. Cabal may not build this library correctly"
-      else Nothing
+sanityCheckLib ml =
+   ml >>= (\l ->
+      toMaybe (null $ exposedModules l)
+              ("Non-empty library, but empty exposed modules list. " ++
+               "Cabal may not build this library correctly"))
 
 checkSanity :: Bool -> String -> Maybe String
-checkSanity False _  = Nothing
-checkSanity True  s  = Just s
+checkSanity = toMaybe
 
 hasMods :: PackageDescription -> Bool
-hasMods pkg_descr
-    | isNothing (library pkg_descr) = null $ executables pkg_descr
-    | otherwise = null (executables pkg_descr)
-                   && null (exposedModules (fromJust (library pkg_descr)))
+hasMods pkg_descr =
+   null (executables pkg_descr) &&
+      maybe True (null . exposedModules) (library pkg_descr)
+
 
 -- ------------------------------------------------------------
 -- * Testing
 -- ------------------------------------------------------------
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
