@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.CoordTrans
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -35,7 +35,6 @@ module Graphics.Rendering.OpenGL.GL.CoordTrans (
    Plane(..), TextureCoordName(..), TextureGenMode(..), textureGenMode
 ) where
 
-import Control.Monad ( liftM )
 import Foreign.ForeignPtr ( ForeignPtr, mallocForeignPtrArray, withForeignPtr )
 import Foreign.Marshal.Alloc ( alloca )
 import Foreign.Marshal.Array ( allocaArray, peekArray, pokeArray, withArray )
@@ -75,6 +74,172 @@ import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
    recordInvalidEnum, recordInvalidValue )
 
 --------------------------------------------------------------------------------
+
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
+
+
+
+
+                                                                                                                                                                          
+
+
+
+
+
+
+
+
+
+
+
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
+
+
+
+
+                                                                                  
+                                                                               
+
+                                                    
+
+
+                                                            
+
+
+                                                          
+
+
+                                                         
+
+
+                                                        
+
+
+                                                           
+
+
+                                                         
+
+
+                                                             
+                              
+
+                                                            
+                             
+
+                                                         
+
+
+                                                         
+
+
+                                                          
+
+
+                                                         
+
+
+                                                           
+
+
+                                                            
+
+
+                                                         
+
+
+                                           
+
+
+                                          
+
+
+                                       
+
+
+                                         
+
+
+                                         
+
+
+                                         
+
+
+                                       
+
+
+                                        
+
+
+                                      
+
+
+                                        
+
+
+                                        
+
+
+                                        
+
+
+                                       
+
+
+                                         
+
+
+                                                                              
+
+
+                                              
+
+
+                                                          
+
+
+                                                          
+
+
+                                            
+
+
+                                                      
+
+
+                                                             
+                              
+
+                                                                      
+                              
+
+
+
+                                                                                                                     
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 --------------------------------------------------------------------------------
 
@@ -368,8 +533,14 @@ multMatrix mat =
 
 data GLmatrix a = GLmatrix MatrixOrder (ForeignPtr a)
 
+
+
+
+
+
    deriving ( Eq, Ord, Show )
 
+
 instance Matrix GLmatrix where
    withNewMatrix order f = do
       fp <- mallocForeignPtrArray 16
@@ -555,8 +726,8 @@ textureGenMode coord =
       (return $ textureCoordNameToEnableCap coord)
       (do mode <- getMode coord
           case mode of
-             EyeLinear'     -> liftM EyeLinear $ getPlane coord EyePlane
-             ObjectLinear'  -> liftM ObjectLinear $ getPlane coord ObjectPlane
+             EyeLinear'     -> fmap EyeLinear $ getPlane coord EyePlane
+             ObjectLinear'  -> fmap ObjectLinear $ getPlane coord ObjectPlane
              SphereMap'     -> return SphereMap
              NormalMap'     -> return NormalMap
              ReflectionMap' -> return ReflectionMap)
