@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.Evaluators
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -43,7 +43,7 @@ module Graphics.Rendering.OpenGL.GL.Evaluators (
    autoNormal
 ) where
 
-import Control.Monad ( liftM, zipWithM_ )
+import Control.Monad ( zipWithM_ )
 import Data.List ( genericLength )
 import Foreign.ForeignPtr ( ForeignPtr, mallocForeignPtrArray, withForeignPtr )
 import Foreign.Marshal.Alloc ( alloca )
@@ -77,6 +77,155 @@ import Graphics.Rendering.OpenGL.GL.VertexSpec (
    Normal3(..), Color4(..), Index1(..) )
 
 --------------------------------------------------------------------------------
+
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
+
+
+
+
+                                                                                  
+                                                                               
+
+                                                    
+
+
+                                                            
+
+
+                                                          
+
+
+                                                         
+
+
+                                                        
+
+
+                                                           
+
+
+                                                         
+
+
+                                                             
+                              
+
+                                                            
+                             
+
+                                                         
+
+
+                                                         
+
+
+                                                          
+
+
+                                                         
+
+
+                                                           
+
+
+                                                            
+
+
+                                                         
+
+
+                                           
+
+
+                                          
+
+
+                                       
+
+
+                                         
+
+
+                                         
+
+
+                                         
+
+
+                                       
+
+
+                                        
+
+
+                                      
+
+
+                                        
+
+
+                                        
+
+
+                                        
+
+
+                                       
+
+
+                                         
+
+
+                                                                              
+
+
+                                              
+
+
+                                                          
+
+
+                                                          
+
+
+                                            
+
+
+                                                      
+
+
+                                                             
+                              
+
+                                                                      
+                              
+
+
+
+                                                                                                                     
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 --------------------------------------------------------------------------------
 
@@ -360,8 +509,14 @@ class Map1 m where
 data (ControlPoint c, Domain d) => GLmap1 c d =
    GLmap1 (MapDescriptor d) (ForeignPtr d)
 
+
+
+
+
+
    deriving ( Eq, Ord, Show )
 
+
 instance Map1 GLmap1 where
    withNewMap1 descriptor act = do
       fp <- mallocForeignPtrArray (totalComponents1 descriptor)
@@ -394,7 +549,7 @@ getMap1 dummyControlPoint = do
       peek2 (,) ptr
    order <- alloca $ \ptr -> do
       glGetMapiv target (marshalGetMapQuery Order) ptr
-      liftM fromIntegral $ peek ptr
+      fmap fromIntegral $ peek ptr
    withNewMap1 (MapDescriptor domain (numComponents dummyControlPoint) order numComp) $
       glGetMapv target (marshalGetMapQuery Coeff)
 
@@ -471,8 +626,14 @@ class Map2 m where
 data (ControlPoint c, Domain d) => GLmap2 c d =
    GLmap2 (MapDescriptor d)  (MapDescriptor d) (ForeignPtr d)
 
+
+
+
+
+
    deriving ( Eq, Ord, Show )
 
+
 instance Map2 GLmap2 where
    withNewMap2 uDescriptor vDescriptor act = do
       fp <- mallocForeignPtrArray (totalComponents2 uDescriptor vDescriptor)
