@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.PerFragment
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -48,7 +48,7 @@ module Graphics.Rendering.OpenGL.GL.PerFragment (
    LogicOp(..), logicOp
 ) where
 
-import Control.Monad ( liftM, liftM2, liftM3 )
+import Control.Monad ( liftM2, liftM3 )
 import Foreign.Marshal.Alloc ( alloca )
 import Foreign.Marshal.Array ( withArrayLen, peekArray, allocaArray )
 import Foreign.Ptr ( Ptr )
@@ -91,6 +91,24 @@ import Graphics.Rendering.OpenGL.GL.VertexSpec ( Color4(..), rgbaMode )
 
 --------------------------------------------------------------------------------
 
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
+
+
+
+
+                                                                                                                                                                          
+
+
+
+
+
+
+
+
+
+
+
+
 --------------------------------------------------------------------------------
 
 scissor :: StateVar (Maybe (Position, Size))
@@ -249,13 +267,14 @@ instance ObjectName QueryObject where
    genObjectNames n =
       allocaArray n $ \buf -> do
         glGenQueriesARB (fromIntegral n) buf
-        liftM (map QueryObject) $ peekArray n buf
+        fmap (map QueryObject) $ peekArray n buf
 
    deleteObjectNames queryObjects =
       withArrayLen (map queryID queryObjects) $
          glDeleteQueriesARB . fromIntegral
 
-   isObjectName = liftM unmarshalGLboolean . glIsQueryARB . queryID
+   isObjectName = fmap unmarshalGLboolean . glIsQueryARB . queryID
+
 
 foreign import ccall unsafe "dynamic" dyn_glGenQueriesARB :: Graphics.Rendering.OpenGL.GL.Extensions.Invoker (GLsizei -> Ptr GLuint -> IO ()) ; glGenQueriesARB :: (GLsizei -> Ptr GLuint -> IO ()) ; glGenQueriesARB = dyn_glGenQueriesARB ptr_glGenQueriesARB ; ptr_glGenQueriesARB :: FunPtr a ; ptr_glGenQueriesARB = unsafePerformIO (Graphics.Rendering.OpenGL.GL.Extensions.getProcAddress ("GL_ARB_occlusion_query or OpenGL 1.5") ("glGenQueriesARB")) ; {-# NOINLINE ptr_glGenQueriesARB #-}
 
