@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.Texturing.Specification
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -37,7 +37,6 @@ module Graphics.Rendering.OpenGL.GL.Texturing.Specification (
    getCompressedTexImage
 ) where
 
-import Control.Monad ( liftM )
 import Foreign.Marshal.Array ( peekArray, allocaArray )
 import Foreign.Ptr ( Ptr )
 import Graphics.Rendering.OpenGL.GL.BasicTypes ( GLint, GLsizei, GLenum )
@@ -60,6 +59,24 @@ import Graphics.Rendering.OpenGL.GL.Texturing.TextureTarget (
 
 --------------------------------------------------------------------------------
 
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
+
+
+
+
+                                                                                                                                                                          
+
+
+
+
+
+
+
+
+
+
+
+
 --------------------------------------------------------------------------------
 
 type Level = GLint
@@ -219,15 +236,21 @@ compressedTextureFormats =
       n <- getInteger1 fromIntegral GetNumCompressedTextureFormats
       allocaArray n $ \buf -> do
          getIntegerv GetCompressedTextureFormats buf
-         liftM (map (CompressedTextureFormat . fromIntegral)) $ peekArray n buf
+         fmap (map (CompressedTextureFormat . fromIntegral)) $ peekArray n buf
 
 --------------------------------------------------------------------------------
 
 data CompressedPixelData a =
      CompressedPixelData !CompressedTextureFormat GLsizei (Ptr a)
 
+
+
+
+
+
    deriving ( Eq, Ord, Show )
 
+
 withCompressedPixelData ::
    CompressedPixelData a -> (GLenum -> GLsizei -> Ptr a -> b) -> b
 withCompressedPixelData
