@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.VertexArrays
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -31,7 +31,6 @@ module Graphics.Rendering.OpenGL.GL.VertexArrays (
    primitiveRestartIndex
 ) where
 
-import Control.Monad ( liftM )
 import Foreign.Marshal.Alloc ( alloca )
 import Foreign.Ptr ( Ptr, nullPtr )
 import Foreign.Storable ( Storable(peek) )
@@ -71,6 +70,24 @@ import Graphics.Rendering.OpenGL.GLU.ErrorsInternal (
 
 --------------------------------------------------------------------------------
 
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
+
+
+
+
+                                                                                                                                                                          
+
+
+
+
+
+
+
+
+
+
+
+
 --------------------------------------------------------------------------------
 
 type NumComponents = GLint
@@ -80,8 +97,14 @@ type Stride = GLsizei
 data VertexArrayDescriptor a =
    VertexArrayDescriptor !NumComponents !DataType !Stride !(Ptr a)
 
+
+
+
+
+
    deriving ( Eq, Ord, Show )
 
+
 noVertexArrayDescriptor :: VertexArrayDescriptor a
 noVertexArrayDescriptor = VertexArrayDescriptor 0 Byte 0 nullPtr
 
@@ -459,7 +482,7 @@ getPrimitiverestartIndex :: IO (Maybe ArrayIndex)
 getPrimitiverestartIndex = do
    state <- get (makeCapability CapPrimitiveRestart)
    if state == Enabled
-      then liftM Just $ getInteger1 fromIntegral GetPrimitiveRestartIndex
+      then fmap Just $ getInteger1 fromIntegral GetPrimitiveRestartIndex
       else return Nothing
 
 setPrimitiverestartIndex :: Maybe ArrayIndex -> IO ()
