@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GL.VertexSpec
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -76,6 +76,172 @@ import Graphics.Rendering.OpenGL.GL.Texturing.TextureUnit (
    TextureUnit(..), marshalTextureUnit, unmarshalTextureUnit )
 
 --------------------------------------------------------------------------------
+
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
+
+
+
+
+                                                                                                                                                                          
+
+
+
+
+
+
+
+
+
+
+
+                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    
+
+
+
+
+                                                                                  
+                                                                               
+
+                                                    
+
+
+                                                            
+
+
+                                                          
+
+
+                                                         
+
+
+                                                        
+
+
+                                                           
+
+
+                                                         
+
+
+                                                             
+                              
+
+                                                            
+                             
+
+                                                         
+
+
+                                                         
+
+
+                                                          
+
+
+                                                         
+
+
+                                                           
+
+
+                                                            
+
+
+                                                         
+
+
+                                           
+
+
+                                          
+
+
+                                       
+
+
+                                         
+
+
+                                         
+
+
+                                         
+
+
+                                       
+
+
+                                        
+
+
+                                      
+
+
+                                        
+
+
+                                        
+
+
+                                        
+
+
+                                       
+
+
+                                         
+
+
+                                                                              
+
+
+                                              
+
+
+                                                          
+
+
+                                                          
+
+
+                                            
+
+
+                                                      
+
+
+                                                             
+                              
+
+                                                                      
+                              
+
+
+
+                                                                                                                     
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 --------------------------------------------------------------------------------
 
@@ -812,13 +978,16 @@ currentColor :: StateVar (Color4 GLfloat)
 currentColor =
    makeStateVar (getFloat4 Color4 GetCurrentColor) color
 
--- The current secondary color (/R/, /G/, /B/, /A/). The initial value is
--- (0, 0, 0, 1). Note that this state variable is significant only when the GL is
--- in RGBA mode.
+-- The current secondary color (/R/, /G/, /B/). The initial value is (0, 0, 0).
+-- Note that this state variable is significant only when the GL is in RGBA
+-- mode.
 
-currentSecondaryColor :: StateVar (Color4 GLfloat)
+currentSecondaryColor :: StateVar (Color3 GLfloat)
 currentSecondaryColor =
-   makeStateVar (getFloat4 Color4 GetCurrentSecondaryColor) color
+   makeStateVar
+      (do Color4 r g b _ <- getFloat4 Color4 GetCurrentSecondaryColor
+          return $ Color3 r g b)
+      secondaryColor
 
 --------------------------------------------------------------------------------
 
@@ -977,6 +1146,7 @@ foreign import ccall unsafe "glColor3ub" glColor3ub ::
 foreign import ccall unsafe "glColor4ub" glColor4ub ::
    GLubyte -> GLubyte -> GLubyte -> GLubyte -> IO ()
 
+
 foreign import ccall unsafe "glColor3ubv" glColor3ubv ::
    Ptr GLubyte -> IO ()
 
