@@ -1,7 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.OpenGL.GLU.Tessellation
--- Copyright   :  (c) Sven Panne 2002-2004
+-- Copyright   :  (c) Sven Panne 2002-2005
 -- License     :  BSD-style (see the file libraries/OpenGL/LICENSE)
 -- 
 -- Maintainer  :  sven.panne@aedion.de
@@ -35,7 +35,7 @@ module Graphics.Rendering.OpenGL.GLU.Tessellation (
    Primitive(..), SimplePolygon(..), tessellate
 ) where
 
-import Control.Monad ( foldM, liftM, unless )
+import Control.Monad ( foldM, unless )
 import Data.IORef ( newIORef, readIORef, writeIORef, modifyIORef )
 import Foreign.Marshal.Alloc ( allocaBytes )
 import Foreign.Marshal.Array ( peekArray, pokeArray )
@@ -134,8 +134,13 @@ marshalTessWinding x = case x of
 
 data AnnotatedVertex v = AnnotatedVertex (Vertex3 GLdouble) v
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 offsetOfProperty :: Storable v => v -> Int
 offsetOfProperty v = alignOffset v (3 * sizeOf x)
    where AnnotatedVertex (Vertex3 x _ _) _ = undefined
@@ -174,8 +179,13 @@ instance Storable v => Storable (AnnotatedVertex v) where
 
 newtype ComplexContour v = ComplexContour [AnnotatedVertex v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 sizeOfComplexContour :: Storable v => ComplexContour v -> Int
 sizeOfComplexContour (ComplexContour vs) =
    length vs * sizeOf (head vs)
@@ -192,8 +202,13 @@ pokeComplexContour ptr (ComplexContour vs) =
 
 newtype ComplexPolygon v = ComplexPolygon [ComplexContour v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 sizeOfComplexPolygon :: Storable v => ComplexPolygon v -> Int
 sizeOfComplexPolygon (ComplexPolygon complexContours) =
    sum (map sizeOfComplexContour complexContours)
@@ -224,8 +239,13 @@ data WeightedProperties v
                         (GLclampf, v)
                         (GLclampf, v)
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 -- | A function combining given vertex properties into a property for a newly
 -- generated vertex
 
@@ -266,15 +286,25 @@ type Tessellator p v
 
 newtype SimpleContour v = SimpleContour [AnnotatedVertex v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 -- | The contours of a complex polygon, represented by one or more
 -- non-intersecting simple contours
 
 newtype PolygonContours v = PolygonContours [SimpleContour v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 extractContours :: Storable v => Tessellator PolygonContours v
 extractContours windingRule tolerance normal combiner complexPoly = do
 
@@ -287,7 +317,7 @@ extractContours windingRule tolerance normal combiner complexPoly = do
           writeIORef vertices []
           modifyIORef contours (SimpleContour (reverse vs) :)
 
-       getContours = liftM (PolygonContours . reverse) (readIORef contours)
+       getContours = fmap (PolygonContours . reverse) (readIORef contours)
 
    withTessellatorObj (PolygonContours [])$ \tessObj -> do
       setTessellatorProperties tessObj windingRule tolerance normal True
@@ -309,14 +339,24 @@ type TriangleVertex v = AnnotatedVertex (v,EdgeFlag)
 data Triangle v
    = Triangle (TriangleVertex v) (TriangleVertex v) (TriangleVertex v)
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 -- | A triangulation of a complex polygon
 
 newtype Triangulation v = Triangulation [Triangle v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 triangulate :: Storable v => Tessellator Triangulation v
 triangulate windingRule tolerance normal combiner complexPoly = do
 
@@ -350,12 +390,22 @@ collectTriangles _            = error "triangles left"
 
 data Primitive v = Primitive PrimitiveMode [AnnotatedVertex v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 newtype SimplePolygon v = SimplePolygon [Primitive v]
 
+
+
+
+
    deriving ( Eq, Ord )
 
+
 tessellate :: Storable v => Tessellator SimplePolygon v
 tessellate windingRule tolerance normal combiner complexPoly = do
 
@@ -372,7 +422,7 @@ tessellate windingRule tolerance normal combiner complexPoly = do
           writeIORef vertices []
           modifyIORef primitives (Primitive beginMode (reverse vs) :)
 
-       getSimplePolygon = liftM (SimplePolygon . reverse) (readIORef primitives)
+       getSimplePolygon = fmap (SimplePolygon . reverse) (readIORef primitives)
 
    withTessellatorObj (SimplePolygon []) $ \tessObj -> do
       setTessellatorProperties tessObj windingRule tolerance normal False
