@@ -1,8 +1,6 @@
-{-# OPTIONS -fffi -cpp #-}
+{-# OPTIONS -cpp #-}
 
 ------------------------------------------------------------------------
--- $Id: Main.hs,v 1.72 2005/03/10 17:58:42 malcolm Exp $
---
 -- Program for converting .hsc files to .hs files, by converting the
 -- file into a C program which is run to generate the Haskell source.
 -- Certain items known only to the C compiler can then be used in
@@ -11,16 +9,78 @@
 --
 -- See the documentation in the Users' Guide for more details.
 
+import Control.Monad		( MonadPlus(..), liftM, liftM2, when )
+import Data.Char		( isAlpha, isAlphaNum, isSpace, isDigit,
+				  toUpper, intToDigit, ord )
+import Data.List		( intersperse, isSuffixOf )
+import System.Cmd		( system, rawSystem )
 import System.Console.GetOpt
+import System.Directory		( removeFile, doesFileExist )
+import System.Environment	( getProgName, getArgs )
+import System.Exit		( ExitCode(..), exitWith )
+import System.IO		( hPutStr, hPutStrLn, stderr )
+
+
+import System.Directory		( findExecutable )
+
+
+
+
+
+
+
+
+
+
+
+
+
+import Paths_hsc2hs		( getDataFileName )
+
+
+
+
+
+
+
+
+
+default_compiler = "gcc"
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
-import System        (getProgName, getArgs, ExitCode(..), exitWith, system)
-import Directory     (removeFile,doesFileExist)
-import Monad         (MonadPlus(..), liftM, liftM2, when)
-import Char          (isAlpha, isAlphaNum, isSpace, isDigit, toUpper, intToDigit, ord)
-import List          (intersperse, isSuffixOf)
-import IO            (hPutStr, hPutStrLn, stderr)
 
-import System.Cmd	 	( rawSystem )
 
 version :: String
 version = "hsc2hs version 0.66\n"
@@ -81,7 +141,6 @@ options = [
         "display this help and exit",
     Option ['V'] ["version"]    (NoArg  Version)
         "output version information and exit" ]
-    
 
 main :: IO ()
 main = do
@@ -90,26 +149,14 @@ main = do
     args <- getArgs
     let (flags, files, errs) = getOpt Permute options args
 
-	-- If there is no Template flag explicitly specified, try
-	-- to find one by looking near the executable.  This only
-	-- works on Win32 or Hugs (getExecDir). On Unix, there's a wrapper 
-	-- script which specifies an explicit template flag.
-    flags_w_tpl <- if any template_flag flags then
-			return flags
-		   else	
-
-			do mb_path <- getExecDir "/Main.hs"
-
-			   add_opt <-
-			    case mb_path of
-			      Nothing   -> return id
-			      Just path -> do
-				let templ = path ++ "/template-hsc.h"
-				flg <- doesFileExist templ
-				if flg 
-				 then return ((Template templ):)
-				 else return id
-		           return (add_opt flags) 
+	-- If there is no Template flag explicitly specified,
+	-- use the file placed by the Cabal installation.
+    flags_w_tpl <-
+	if any template_flag flags then
+	    return flags
+	  else do
+	    templ <- getDataFileName "template-hsc.h"
+	    return (Template templ : flags)
     case (files, errs) of
         (_, _)
             | any isHelp    flags_w_tpl -> bye (usageInfo header options)
@@ -133,7 +180,7 @@ die :: String -> IO a
 die s = hPutStr stderr s >> exitWith (ExitFailure 1)
 
 processFile :: [Flag] -> String -> IO ()
-processFile flags name 
+processFile flags name
   = do let file_name = dosifyPath name
        s <- readFile file_name
        case parser of
@@ -466,7 +513,7 @@ splitExt name =
 
 output :: [Flag] -> String -> [Token] -> IO ()
 output flags name toks = do
-    
+
     (outName, outDir, outBase) <- case [f | Output f <- flags] of
         [] -> if not (null ext) && last ext == 'c'
                  then return (dir++base++init ext,  dir, base)
@@ -482,36 +529,44 @@ output flags name toks = do
             (base, _)    = splitExt file
             in return (f, dir, base)
         _ -> onlyOne "output file"
-    
+
     let cProgName    = outDir++outBase++"_hsc_make.c"
         oProgName    = outDir++outBase++"_hsc_make.o"
         progName     = outDir++outBase++"_hsc_make"
 
+
+
+
+
 	outHFile     = outBase++"_hsc.h"
         outHName     = outDir++outHFile
         outCName     = outDir++outBase++"_hsc.c"
-	
+
 	beVerbose    = any (\ x -> case x of { Verbose -> True; _ -> False}) flags
 
     let execProgName
             | null outDir = dosifyPath ("./" ++ progName)
             | otherwise   = progName
-    
+
     let specials = [(pos, key, arg) | Special pos key arg <- toks]
-    
+
     let needsC = any (\(_, key, _) -> key == "def") specials
         needsH = needsC
-    
+
     let includeGuard = map fixChar outHName
             where
             fixChar c | isAlphaNum c = toUpper c
                       | otherwise    = '_'
 
     compiler <- case [c | Compiler c <- flags] of
-        []  -> return "gcc"
+        []  -> do
+	    mb_path <- findExecutable default_compiler
+	    case mb_path of
+		Nothing -> die ("Can't find "++default_compiler++"\n")
+		Just path -> return path
         [c] -> return c
         _   -> onlyOne "compiler"
-    
+
     linker <- case [l | Linker l <- flags] of
         []  -> return compiler
         [l] -> return l
@@ -525,42 +580,30 @@ output flags name toks = do
         outHsLine (SourcePos name 0)++
         concatMap outTokenHs toks++
         "    return 0;\n}\n"
-    
+
     -- NOTE: hbc compiles "[() | NoCompile <- flags]" into wrong code,
     -- so we use something slightly more complicated.   :-P
     when (any (\x -> case x of NoCompile -> True; _ -> False) flags) $
        exitWith ExitSuccess
 
-    
-    compilerStatus <- rawSystemL beVerbose compiler
+    rawSystemL ("compiling " ++ cProgName) beVerbose compiler
 	(  ["-c"]
         ++ [f | CompFlag f <- flags]
         ++ [cProgName]
         ++ ["-o", oProgName]
 	)
-
-    case compilerStatus of
-        e@(ExitFailure _) -> exitWith e
-        _                 -> return ()
     removeFile cProgName
-    
-    linkerStatus <- rawSystemL beVerbose linker
+
+    rawSystemL ("linking " ++ oProgName) beVerbose linker
         (  [f | LinkFlag f <- flags]
         ++ [oProgName]
         ++ ["-o", progName]
 	)
-
-    case linkerStatus of
-        e@(ExitFailure _) -> exitWith e
-        _                 -> return ()
     removeFile oProgName
-    
-    progStatus <- systemL beVerbose (execProgName++" >"++outName)
+
+    rawSystemWithStdOutL ("running " ++ execProgName) beVerbose execProgName [] outName
     removeFile progName
-    case progStatus of
-        e@(ExitFailure _) -> exitWith e
-        _                 -> return ()
-    
+
     when needsH $ writeFile outHName $
         "#ifndef "++includeGuard++"\n" ++
         "#define "++includeGuard++"\n" ++
@@ -575,22 +618,41 @@ output flags name toks = do
         concatMap outFlagH flags++
         concatMap outTokenH specials++
         "#endif\n"
-    
+
     when needsC $ writeFile outCName $
         "#include \""++outHFile++"\"\n"++
         concatMap outTokenC specials
 	-- NB. outHFile not outHName; works better when processed
 	-- by gcc or mkdependC.
 
-rawSystemL :: Bool -> String -> [String] -> IO ExitCode
-rawSystemL flg prog args = do
-  when flg $ hPutStrLn stderr ("Executing: " ++ prog ++ concat (map (' ':) args))
-  rawSystem prog args
+rawSystemL :: String -> Bool -> FilePath -> [String] -> IO ()
+rawSystemL action flg prog args = do
+  let cmdLine = prog++" "++unwords args
+  when flg $ hPutStrLn stderr ("Executing: " ++ cmdLine)
 
-systemL :: Bool -> String -> IO ExitCode
-systemL flg s = do
-  when flg (hPutStrLn stderr ("Executing: " ++ s))
-  system s
+  exitStatus <- system cmdLine
+
+
+
+  case exitStatus of
+    ExitFailure _ -> die $ action ++ " failed\ncommand was: " ++ cmdLine ++ "\n"
+    _             -> return ()
+
+rawSystemWithStdOutL :: String -> Bool -> FilePath -> [String] -> FilePath -> IO ()
+rawSystemWithStdOutL action flg prog args outFile = do
+  let cmdLine = prog++" "++unwords args++" >"++outFile
+  when flg (hPutStrLn stderr ("Executing: " ++ cmdLine))
+
+  exitStatus <- system cmdLine
+
+
+
+
+
+
+  case exitStatus of
+    ExitFailure _ -> die $ action ++ " failed\ncommand was: " ++ cmdLine ++ "\n"
+    _             -> return ()
 
 onlyOne :: String -> IO a
 onlyOne what = die ("Only one "++what++" may be specified\n")
@@ -778,7 +840,7 @@ outCLine (SourcePos name line) =
 outHsLine :: SourcePos -> String
 outHsLine (SourcePos name line) =
     "    hsc_line ("++show (line + 1)++", \""++
-    showCString (snd (splitName name))++"\");\n"
+    showCString name++"\");\n"
 
 showCString :: String -> String
 showCString = concatMap showCChar
@@ -801,22 +863,15 @@ showCString = concatMap showCChar
                       intToDigit (ord c          `mod` 8)]
 
 -----------------------------------------
---	Cut and pasted from ghc/compiler/SysTools
+-- Modified version from ghc/compiler/SysTools
 -- Convert paths foo/baz to foo\baz on Windows
 
-dosifyPath, unDosifyPath :: String -> String
+subst :: Char -> Char -> String -> String
 
-dosifyPath xs = xs
-unDosifyPath xs = xs
 
-getExecDir :: String -> IO (Maybe String)
--- (getExecDir cmd) returns the directory in which the current
---	  	    executable, which should be called 'cmd', is running
--- So if the full path is /a/b/c/d/e, and you pass "d/e" as cmd,
--- you'll get "/a/b/c" back as the result
 
-getExecDir cmd
-  = do
-	s <- getProgName
-	return (Just (reverse (drop (length cmd) (reverse (unDosifyPath s)))))
+subst _ _ = id
 
+
+dosifyPath :: String -> String
+dosifyPath = subst '/' '\\'
